Class: DBus::RemoteService
- Inherits:
-
Object
- Object
- DBus::RemoteService
- Defined in:
- lib/dbus.rb
Overview
A remote service providing objects.
A service is typically a process or application that provides remote objects, but can also be the broadcast service that receives signals from all applications on the Bus.
Instance Method Summary collapse
-
#get_bus ⇒ Object
Bus associated with this service.
-
#get_object(object_path, interface) ⇒ Object
Creates an object for the specified
object_path. -
#get_service_name ⇒ Object
The service name.
-
#initialize(bus, service_name) ⇒ RemoteService
constructor
A new instance of RemoteService.
Constructor Details
#initialize(bus, service_name) ⇒ RemoteService
Returns a new instance of RemoteService.
336 337 338 339 |
# File 'lib/dbus.rb', line 336 def initialize(bus, service_name) @bus = bus @service_name = service_name end |
Instance Method Details
#get_bus ⇒ Object
Bus associated with this service
342 343 344 |
# File 'lib/dbus.rb', line 342 def get_bus @bus end |
#get_object(object_path, interface) ⇒ Object
Creates an object for the specified object_path
352 353 354 |
# File 'lib/dbus.rb', line 352 def get_object(object_path, interface) RemoteObject.new(self, object_path, interface) end |
#get_service_name ⇒ Object
The service name
347 348 349 |
# File 'lib/dbus.rb', line 347 def get_service_name @service_name end |